# @(#)06  1.5 src/cmdline/staging/scripts/fim_base.py, amws.cmdline, fim611, 060808b 5/4/06 20:19:54
# (C) Copyright International Business Machines Corporation 2005. All rights 
# reserved.  U.S. Government Users Restricted Rights -- Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

import sys
from java.io import File, FileInputStream, BufferedReader, InputStreamReader
from java.lang import String
from jarray import array
from java.util import StringTokenizer
from com.tivoli.am.fim.middlemanager import FIMDomain, WASSecurityInfo 
from com.tivoli.am.fim.middlemanager.rtconfig import RuntimeNodesManager
from com.tivoli.am.fim.middlemanager.util import MgmtUtils
from com.tivoli.am.fim.mgmt.client.sts import ModuleChain
from com.tivoli.am.fim.middlemanager.ssps import Federation, FederationManager
from com.tivoli.am.fim.middlemanager.tokenservice import SecurityTokenManager
from com.tivoli.am.fim.logging import FIMLoggerFactory
from com.tivoli.am.fim.logging import FIMLogger
from com.tivoli.am.fim.logging import FIMLevel
from com.tivoli.am.fim.i18n.msgs import CmdLineMessages
from java.lang import Object
from com.tivoli.am.fim.logging import FIMLoggerConstants

######
# Global variables
#
######
host = "Host"
port = "Port"
domain_name = "Domain"
cluster_name = "Cluster"
domain_required_keys = [host, port, domain_name, cluster_name]

security_enabled = "SecurityEnabled"
admin_user = "AdminUser"
admin_pwd = "AdminPasswd"
trusted_keystore_path = "TrustedKeystorePath"
trusted_keystore_pwd = "TrustedKeystorePasswd"
was_sec_required_keys = [admin_user, admin_pwd, trusted_keystore_path, trusted_keystore_pwd]
FIM_TRUE = 1
FIM_FALSE = 0

# Set up FIMLogger
component_name = "StagingTools"
script_name = "fim_base.py"
method_name = script_name
class_name = "com.tivoli.am.fim.cmdline." + script_name[0:-3]
#FIMLoggerFactory.enableStdout(FIMLevel.ALL)
FIMLoggerFactory.setPreferredFIMLoggerImpl(FIMLoggerConstants.FIMLOGGER_BASE_IMPL)
log = FIMLoggerFactory.getFIMLogger(component_name, class_name, CmdLineMessages.CLASS_NAME)

##########
# get_was_security_object
#
# Takes a dictionary, equivalent to a hashtable in other languages.
# The function uses the values in this table to create WASSecurityInfo object.
##########
def get_was_security_object(props):
    wsSec = WASSecurityInfo()

    for key in was_sec_required_keys:
        if not props.has_key(key):
            return None

    wsSec.setAdminUser(props[admin_user])
    wsSec.setAdminPasswd(props[admin_pwd])
    wsSec.setTrustedKeystorePath(props[trusted_keystore_path])
    wsSec.setTrustedKeystorePasswd(props[trusted_keystore_pwd])
    return wsSec


######
# commit_session
#
# Commits a management session to the management service. All configuration
# is written to disk, and the server must be restarted.
#
# manager: any Java object whose parent class is com.tivoli.am.fim.middlemanager.Manager
######
def commit_session(manager, show_message=FIM_TRUE):
    manager.commitSession()
    
    if show_message:
        # "Changes saved to configuration repository."
        # "Please restart WebSphere before trying to use the TFIM Runtime."
        log.logMessage(FIMLevel.INFO, method_name, CmdLineMessages.restart_websphere)
    
    return


#######
# get_domain_instance
#
# Creates an instance of the FIMDomain object using the given parameters
# notice the was_security_prop_file argument in the function signature is
# assigned to None (the Jython equivalent of Java's null).  Jython provides 
# assignment in the function signature to allow for default values.  When
# a user invokes this function, she can do so without providing that argument
# at all and it will automatically receive the value None.
#
########
def get_domain_instance(props):
    for key in domain_required_keys:
        if not props.has_key(key):
            return None

    domain = FIMDomain(props[domain_name], props[host], props[port], props[cluster_name])
    
    if(props[security_enabled] == "true" or props[security_enabled] == "yes"):
        ws_info = get_was_security_object(props)
        if ws_info != None:
            domain.setWasSecurityInfo(ws_info)

#    rtMgr = RuntimeNodesManager.getInstance("get_domain_instance", domain)
#    if not rtMgr.getCurrentVersion():
#        log.logMessage(FIMLevel.INFO, method_name, CmdLineMessages.no_domains_exist)
#        raise SystemExit
#        
    return domain


######
# read_properties_file
#
# reads a file in the format of Java properties (key=value) and inserts entries into a dictionary
######
def read_properties_file(propsFileName):
    # Set up FIMLogger
    method_name = "read_properties_file"

    try:
        propsFile = open(propsFileName, 'r')
    except IOError:
        inserts = array([propsFileName], Object)
        log.logMessage(FIMLevel.INFO, method_name, \
            CmdLineMessages.properties_file_not_found, inserts)
        raise SystemExit

    props = {}
    for line in propsFile.readlines():
        # skip comments and blank lines
        if line.lstrip().startswith("#") or not len(line.strip()):
            continue
        (key, value) = line.strip().split("=")        
        props[key] = value
    
    propsFile.close()
    
    return props


######
# get_chain_id
#
# given a chain name, get the chain id
# assumes that chain names are unique
#
# returns the chain id if found, None otherwise
######
def get_chain_id(stsMgr, chain_name):
    moduleChainList = stsMgr.listModuleChains()
    for mc in moduleChainList:
        if mc.getName() == chain_name:
            return mc.getId()
    
    return None


######
# properties_to_fim_values
#
# converts a dictionary of string->string mappings to string->string[]
# returns a new dictionary -- the original remains unchanged
######
def properties_to_fim_values(origProps):
    props = {}
    
    array = Array.newInstance(String, 1)
    for k, v in origProps.items():
        Array.set(array, 0, v)
        props[k] = array
    
    return props

######
# jython_to_java_array
#
# converts a Jython list to a Java Array of Objects
# returns the Java Array
######
def jython_to_java_array(jylist):
    size = len(jylist)
    array = Array.newInstance(Object, size)
    i = 0
    while i < size:
        Array.set(array, i, jylist[i])
        i += 1
    
    return array

######
# prepare_mgr_objs
#
# Takes a dictionary of domain properties and returns a FederationManager
# instance and a SecurityTokenManager instance.
######
    
def prepare_mgr_objs(domain_props, caller_id):
    # Set up FIMLogger
    method_name = "prepare_mgr_objs"

    domain = get_domain_instance(domain_props)
    fed_mgr = FederationManager.getInstance(caller_id, domain)
    sec_tok_mgr = SecurityTokenManager.getInstance(caller_id, domain)
    return (fed_mgr, sec_tok_mgr)

######
# create_set_code
#
# Takes a method name and generates the Jython code to invoke the method with
# one argument
######
    
def create_set_code(method_name):
    return "obj.%s(arg)"%method_name

######
# create_get_code
#
# Takes a method name and generates the Jython code to invoke the method with
# no arguments.
######
    
def create_get_code(method_name):
    return "obj.%s()"%method_name

######
# run_set_code
#
# Takes the Jython code generated by create_set_code(), substitutes 
# values for the obj and arg strings, and evaluates the resulting code
######
    
def run_set_code(code, object, argument):
    return eval(code, {"obj":object, "arg":argument})
    

######
# run_get_code
#
# Takes the Jython code generated by create_set_code(), substitutes 
# value for the obj string, and evaluates the resulting code
######
    
def run_get_code(code, object):
    return eval(code, {"obj":object})

######
# print_dictionary
#
# Takes a dictionary and print out key=value
######

def print_dictionary(dictionary):
    keys = dictionary.keys()
    keys.sort()
    for key in keys:
        print "%s=%s"%(key, dictionary[key])

######
# get_key_from_value
#
# Checks to see if the value given is a value in the dictionary
# and returns the associated key if it is.
######

def get_key_from_value(dictionary, value):
    # Set up FIMLogger
    method_name = "get_key_from_value"
    
    keys = dictionary.keys()
    values = dictionary.values()
    if  values.count(value)==1:
        for key in keys:
            if dictionary[key] == value:
                return key
    else:
        return None
    
